/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/********************************************************************************
 *
 *  DRV84xx.c - Definition file for utility functions
 *  DRV843xEVM_FIRMWARE
 *
 *******************************************************************************/

/********************************************************************************
 * HEADER FILES
 *******************************************************************************/
#include <drv84xx/include/drv84xx.h>
#include <stepper.h>
#include "uart.h"
/********************************************************************************
 * MACROS
 *******************************************************************************/
#pragma DATA_SECTION(gDRV843x_Obj,".drvVarSection");
volatile DRV843x_Device_Obj_t gDRV843x_Obj;

/********************************************************************************
 * DATA
 *******************************************************************************/

/********************************************************************************
 * LOCAL ROUTINES
 *******************************************************************************/

/*
 * Clock Initialization
 */
static inline void clock_Init(void)
{
     DCOCTL  = CALDCO_8MHZ;          // Set DCO to 8MHz

     BCSCTL1 = CALBC1_8MHZ;
     BCSCTL3 = LFXT1S_2;             // ACLK = VLO
}

/*
 * GPIO Initialization. This block configures all the DRV843x PINs.
 */
static inline void gpio_Init(void)
{

    // Note: All unused ports should be set to either output or input with
    //       pullup/down enabled

       P1DIR = BIT7 | BIT6 | BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | nSLEEP;       // All OUTPUT
       P1OUT = BIT7 | BIT6 | BIT5 | BIT4 | BIT3 | BIT2 | BIT1;                // nSLEEP low, others high

       P2DIR = BIT6 | BIT3 | STL_MODE | M1_PD;                                // set all pins as outputs except nFAULT, TRQ_PD, STL_MODE_PD and STL_REP
       P2OUT = nFAULT | STL_REP | BIT6;                                       // MSP430 pullup resistors to high on nFAULT and STL_REP and STL_REP_PD = 1
       P2REN = nFAULT | STL_REP;                                              // Enable MSP430 pullup resistors on nFault and STL_REP

       P3DIR = BIT7 | BIT6 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;                // All OUTPUT except bit 5 (RXD)
       P3OUT = 0x00;                                                          // All low

       P4DIR = BIT7 | BIT6 | BIT5 | ENABLE | DIR | STEP | M1 | BIT0;          // All OUTPUTS
       P4OUT = 0x00;                                                          // All low

       P5DIR = BIT7 | BIT6 | BIT5 | BIT4 | BIT3 | BIT2 | BIT1 | BIT0;         // All OUTPUT
       P5OUT = 0x00;                                                          // All low

       P6DIR = M0 | VREF | STATUS;                                            // Only M0, VREF, and STATUS are OUTPUTS
       P6OUT = 0x00;                                                          // All low
       P6SEL = BIT7;                                                          // DAC1 output on P6.7

       P7DIR = 0xFF;                                                          // All OUTPUT
       P7OUT = 0x00;                                                          // All low

       P8DIR = 0xFF;                                                          // All OUTPUT
       P8OUT = 0x00;                                                          // All low

       PADIR = 0xFF;                                                          // All OUTPUT
       PAOUT = 0x00;                                                          // All low

       // Configure DAC12
       DAC12_1CTL = DAC12SREF_3 | DAC12CALON | DAC12IR | DAC12AMP_7;          // DAC1 on P6.7, Select VeREF, 1x, and Amp high speed/high current
       while (DAC12_1CTL & DAC12CALON);                                       // Wait until calibration complete
       DAC12_1CTL = DAC12SREF_3 | DAC12IR ;                                   // DAC1 on P6.7, 1x, output HiZ
       DAC12_0CTL = DAC12SREF_3 | DAC12CALON | DAC12IR | DAC12AMP_7;          // DAC0 on P6.6, Select VeREF, 1x, and Amp high speed/high current
       while (DAC12_0CTL & DAC12CALON);                                       // Wait until calibration complete

       set_ENABLE_lo;                                                         // DRV outputs disabled
      _delay_cycles(8000);                                                    // settling delay
}

/*
 * Setup Watchdog Timer as an Interval Timer(250ms)
 * Timer Interval to Blink the STATUS LED(P2.7)
 */
static inline void watchdogTimer_Init(void)
{
//  WDTCTL = WDT_ADLY_250;             /** < Configure time interval*/
  WDTCTL = WDT_MDLY_32;               // Set Watchdog Timer interval to 4ms
  IE1 |= WDTIE;                       // Watchdog Interrupt enable
}

/*******************************************************************************
* API IMPLEMENTATION
*******************************************************************************/
void drv843x_init()   // Initialize GUI variables
{

    gDRV843x_Obj.Firmware_Version = FW_VERSION;
    gDRV843x_Obj.nFault = 1;
    gDRV843x_Obj.nSleep = 0;
    gDRV843x_Obj.STLREP = 0;
    gDRV843x_Obj.gEnable = 1;
    gDRV843x_Obj.ClrFlt = 0;
    gDRV843x_Obj.StallRep = 1;
    gDRV843x_Obj.MicroSTEP = 6;
    gDRV843x_Obj.DeviceID = 8436;
    gDRV843x_Obj.StallMode = 0;
//    gDRV843x_Obj.TRQCNTvalue = 0;
    gDRV843x_Obj.TRQCNTvalue0 = 0;
    gDRV843x_Obj.TRQCNTvalue1 = 0;
    gDRV843x_Obj.TRQCNTvalue2 = 0;

    switch(P6IN && (RDSON2+RDSON1+RDSON0))
    {
        case 0x0:
            gDRV843x_Obj.RDSON=1200;
            break;

        case RDSON0:
            gDRV843x_Obj.RDSON=900;
            break;

        case RDSON1:
            gDRV843x_Obj.RDSON=550;
            break;

        case RDSON1+RDSON0:
            gDRV843x_Obj.RDSON=350;
            break;

        case RDSON2:
            gDRV843x_Obj.RDSON=175;
            break;

        default:
            gDRV843x_Obj.RDSON=1200;
            break;
    }

    // Determine INTERFACE state=1*INTERFACE(HI-Z = STEP/DIR); 0 (PH/EN:DRV84xxE); 2*INTERFACE (PWM:DRV84xxP)
    char INTERFACE_state = 0;
    P6OUT |= INTERFACE;               // input pullup
    P6REN |= INTERFACE;               // enable input resistor
    INTERFACE_state += P6IN & INTERFACE;  // store state
    P6OUT &= ~INTERFACE;              // input pulldown
    INTERFACE_state += P6IN & INTERFACE;  // store state
    P6REN &= ~INTERFACE;              // disable input resistor

}

void mcu_init()
{
  // Stop Watchdog Timer
  WDTCTL = WDTPW | WDTHOLD;

  // Setup CLKs
  clock_Init();
  uartInit();
  gpio_Init();
  watchdogTimer_Init();
}
