/* --COPYRIGHT--,BSD
 * Copyright (c) 2016,2019, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/********************************************************************************
 *
 *  stepper.c - Definition file for functions related to stepper motion control
 *  DRV843x_Example FIRMWARE
 *  10/31/2020
 *
 *******************************************************************************/

/********************************************************************************
 * HEADER FILES
 *******************************************************************************/
#include <stepper.h>

/********************************************************************************
 * MACROS
 *******************************************************************************/
#pragma DATA_SECTION(gStepper_Obj,".stpVarSection");
volatile Stepper_Driving_Obj_t gStepper_Obj;

/*****************************************************************************
* Define Global Variables in stepper.h
******************************************************************************/
 /* LOCAL ROUTINES
 *******************************************************************************/
/*
 * TimerA Initialization for PWM
 * From MSP430x261x_ta_13.c in MSPWARE
 */
void stepper_Init(void)
{

    set_ENABLE_lo;

    // configure timer A
    TACTL =   TASSEL_2 |ID_3 | MC_0;

    // configure timer B
    // select SMCLK, divide by 1, stop
    TBCTL =  CNTL_0 | TBSSEL_2 |ID_0 | MC_0;

    // Initialize TimerB PWM on P4.2
    P4DIR |= STEP;
    P4SEL |= STEP;                            // STEP pin GPIO select

    // Set Default GUI Variables Values
    SetDefaultGUIVals();

    gStepper_Obj.gDIR = false;

    // Load GPIO Values
    UpdateDeviceControls();
    UpdateGUIControls();

    // set boolean defaults
    LastCommandWasRunSteps = false;
    MoveSteps = false;
    UpdateSteppingRateTMR = false;
    temp_reciprocation  = false;

}

void SetDefaultGUIVals()
{
    last_reciprocation_state = 0;

    gStepper_Obj.MOTOR_CMD_ACTIVE1 = false;
    gStepper_Obj.MOTOR_CMD_ACTIVE2 = false;
    gStepper_Obj.MOTOR_CMD_ACTIVE3 = false;
    gStepper_Obj.STARTING_SPEED = DEFAULT_START_SPEED;     // initial speed in pulses per second when starting motor
    gStepper_Obj.TARGET_SPEED = DEFAULT_TARGET_SPEED;      // speed of motor once running
    gStepper_Obj.ACCEL_RATE = DEFAULT_ACCEL_RATE;          // acceleration rate from starting speed to target speed
    gStepper_Obj.STOPPING_SPEED = DEFAULT_STOPPING_SPEED;  // speed to stop motor once deceleration has reached this point
    gStepper_Obj.NUM_STEPS = DEFAULT_NUM_STEPS;            // number of steps to move when manually stepping
    gStepper_Obj.STEPS_2_STOP = DEFAULT_STEPS_2_STOP;      // number of steps used to stop the motor when manually stepping
    gStepper_Obj.IFS = DEFAULT_IFS;
    IFS_previous = DEFAULT_IFS+1;
    gStepper_Obj.RUN_STEPPER = false;                      // Auto run enabled
    gStepper_Obj.MANUAL_ADVANCE = false;                   // Manually advance motor
    gStepper_Obj.RECIPROCATION = true;

}

//
// routine to communicate between GUI and device
// reads GUI variables and updates the GPIOs

void UpdateDeviceControls()
{
    // toggle status if count is reached
    if (LED_TOGGLE_CNT >= LED_TIMEOUT)
    {
        toggle_status_led;
        LED_TOGGLE_CNT = 0;
    }

    // from GUI to MSP430
    // update the GPIO Pins from Global Variables

    // during Fault Condition
    // read from Micro and pass to GUI
    if (read_nFAULT_pin == 0)
        gDRV843x_Obj.nFault = 0;
    else
        gDRV843x_Obj.nFault = 1;

    // read STL_REP pin and report to GUI
    if (read_StlRep_pin == 0)
        gDRV843x_Obj.STLREP = 0;                   // display GUI Stall Report Status green
    else
        gDRV843x_Obj.STLREP = 1;                   // display GUI Stall Report Status red

    // read asleep and awake GUI settings
    if (gDRV843x_Obj.nSleep == 1)
        {
        set_nSLEEP_hi;
        }
    else
        {
        set_nSLEEP_lo;
        }

    // DIR
    if (gStepper_Obj.gDIR == 1)
        set_DIR_hi;
    else
        set_DIR_lo;

    // ENABLE  -- set port high to enable device
    if (gDRV843x_Obj.gEnable == 0)
    {
        set_ENABLE_lo;
    }
    else
    {
        if (gDRV843x_Obj.OCP_latched == 0)
            {
                set_ENABLE_hi;
            }
        else    set_ENABLE_zz;
    }

// global variable gDRV843x_Obj.ClrFlt to read GUI 'Clear' button and issue an ~26us 'nSLEEP Reset Pulse' to clear faults
    if (gDRV843x_Obj.ClrFlt == 1)
    {
        clear_fault_loop = 0;
        if (read_nFAULT_pin == 0)
        {
        while (clear_fault_loop <1)
        {
            clear_fault_loop ++;
            set_nSLEEP_lo;                               // pulse low nSLEEP
            _delay_cycles(200);                          // ~26us
            set_nSLEEP_hi;                               // nSLEEP back to high
        }
        gDRV843x_Obj.ClrFlt = 0;
        }
    }

// global variable gDRV843x_Obj.StallMode for Stall Mode setting using the GUI
// global variable gDRV843x_Obj.StallRep for enable/disable Stall Fault Reporting
// sets Stall Detection Operating modes based on STL_MODE bit setting
//     0 (StallMode = 0) -- set MSP430 TRQ_CNT pin as input to allow ADC12 A7 analog measurement and external measurement on the device pin, monitor STL_REP
//     HiZ (StallMode = 1) -- set MSP430 TRQ_CNT pin as input to allow ADC12 A7 analog measurement and external measurement on the device pin, monitor STL_REP
//     pin for Learn success - starts low then goes high, then goes low to signal successful learning after a forced stall
//     1 (StallMode = 2) -- set MSP430 TRQ_CNT pin as DAC12_1 output to input stall threshold to the device STL_TH pin, monitor STL_REP
// DAC1 output P6.5
    switch(gDRV843x_Obj.StallMode)
             {
             case INTERNAL_THRESHOLD:                // STL_MODE = 0
                 set_StallMode_lo;                   // stall mode set to torque count mode
                 set_TRQCNT_to_input;                // MSP430 TRQ_CNT analog input - DAC12_1 set to high impedance
                 _delay_cycles(80);              // settling delay
                 ADC12CTL0 &= ~ENC;                  // ENC =0 to allow ADC12 control bits setting
                 ADC12CTL0 = SHT0_12 + ADC12ON;      // ADC12 on and 256 ADC12CLK cycles sample and hold time
                 ADC12CTL1 = SHP;                    // enable sample timer
                 ADC12MCTL0 = INCH_7;                // AVCC & AVSS as reference (3.3V), A7 analog input
                 ADC12CTL0 |= ENC;                   // enable conversion
                 ADC12CTL0 |= ADC12SC;               // sampling and conversion start
                 while (ADC12CTL1 & ADC12BUSY);      // wait until conversion is done
                 ADC12CTL0 &= ~ENC;                  // ENC =0 to allow ADC12 control bits setting
                 ADC12CTL0 = 0;                      // ADC12 off
                 gDRV843x_Obj.TRQCNTvalue0 = (ADC12MEM0*0.00081); // ADC12counts/1241 reference scaling to display volts
                 if (gDRV843x_Obj.StallRep == 1)
                 {
                     enable_StlRep;                  // enable GUI Stall Reporting
                 }
                 else
                 {
                     disable_StlRep;                 // disable Stall Reporting
                 }
                 break;
             case LEARN:                             // STL_MODE = HiZ
                 enable_StlRep;                      // enable GUI Stall Reporting
                 set_StallMode_hiz;                  // stall mode set to learning mode
                 set_TRQCNT_to_input;                // MSP430 TRQ_CNT analog input - DAC12_1 set to high impedance
                 if (read_StlRep_pin == 0)
                 {
                     _delay_cycles(80);              // settling delay
                     ADC12CTL0 &= ~ENC;              // ENC =0 to allow ADC12 control bits setting
                     ADC12CTL0 = SHT0_12 + ADC12ON;  // ADC12 on and 256 ADC12CLK cycles sample and hold time
                     ADC12CTL1 = SHP;                // enable sample timer
                     ADC12MCTL0 = INCH_7;            // AVCC & AVSS as reference (3.3V), A7 analog input
                     ADC12CTL0 |= ENC;               // enable conversion
                     ADC12CTL0 |= ADC12SC;           // sampling and conversion start
                     while (ADC12CTL1 & ADC12BUSY);  // wait until conversion completed
                     ADC12CTL0 &= ~ENC;              // ENC =0 to allow ADC12 control bits setting
                     ADC12CTL0 = 0;                  // ADC12 off
                     gDRV843x_Obj.TRQCNTvalue1 = (ADC12MEM0*0.00081); // ADC12counts/1241 reference scaling to display volts
                 }
                 gDRV843x_Obj.TRQCNTvalue2 = gDRV843x_Obj.TRQCNTvalue1; // for stall threshold mode
                 break;
             case EXTERNAL_THRESHOLD:                // STL_MODE = 1
                 set_StallMode_hi;                   // stall mode set to stall threshold mode
                 set_TRQCNT_value (1241*gDRV843x_Obj.TRQCNTvalue2); // output stall threshold voltage on DAC12_1 output
                 if (gDRV843x_Obj.StallRep == 1)
                 {
                     enable_StlRep;                  // enable GUI Stall Reporting
                 }
                 else
                 {
                     disable_StlRep;                 // disable GUI Stall Reporting
                 }
                 break;
             }

#ifdef TOFF
    //     TOFF
    switch (gDRV843x_Obj.gToff)
           {
           case TOFF_7u:                            // 7uS TOFF time
               set_TOFF_7us;
               break;
           case TOFF_16u:                           // 16uS TOFF time
               set_TOFF_16us;
               break;
           case TOFF_24u:                           // 24uS TOFF time
               set_TOFF_24us;
               break;
           case TOFF_32u:                           // 32uS TOFF time
               set_TOFF_32us;
               break;
             }

    //     DECAY MODE
   switch (gDRV843x_Obj.gDecay)
            {
            case STDD_STDD:                         // Smart Tune Dynamic Control
                set_dynamic_decay;
                break;
            case STRC_STRC:                         // Smart Tune Ripple Control
                set_ripple_control;
                break;
            case MIX30_MIX30:                       // Mixed Decay 30% fast
                set_mixed30_mixed30;
                break;
            case SLOW_MIX30:                        // Increasing steps Slow; Decreasing steps Mixed decay30% fast
                set_slow_mixed30;
                break;
            case MIX60_MIX60:                       // Mixed Decay 60% fast
                set_mixed60_mixed60;
                break;
            case SLOW_SLOW:                         // Slow Decay
                set_slow_slow;
                break;
            }
#endif

// MICROSTEP settings
  switch (gDRV843x_Obj.MicroSTEP)
           {
           case FULLSTEP_100:                         /* Full step (2-phase excitation) with 100% current */
               set_fullstep_100;
               break;
           case FULLSTEP_71:                          /* Full step (2-phase excitation) with 71% current */
               set_fullstep_71;
               break;
           case NONCIRCULAR_HALFSTEP:                 /* Non-circular 1/2 step */
               set_noncircular_halfstep;
               break;
           case HALFSTEP:                             /* half step*/
               set_halfstep;
               break;
           case MICROSTEP_4:                          /* 1/4 step*/
               set_microstep_4;
               break;
           case MICROSTEP_8:                          /* 1/8 step*/
               set_microstep_8;
               break;
           case MICROSTEP_16:                         /* 1/16 step*/
               set_microstep_16;
               break;
           case MICROSTEP_32:                         /* 1/32 step*/
               set_microstep_32;
               break;
           case MICROSTEP_64:                         /* 1/64 step*/
               set_microstep_64;
               break;
           case MICROSTEP_128:                        /* 1/128 step*/
               set_microstep_128;
               break;
           case MICROSTEP_256:                        /* 1/256 step*/
               set_microstep_256;
               break;
           }
}

//
// routine to communicate between device and GUI
// reads MSP430 GPIOs and updates the GUI
void UpdateGUIControls()
{
    // Enable Motor control buttons
    if (gDRV843x_Obj.gEnable == 0 || gDRV843x_Obj.nSleep == 0)
    {
        // when disabled reset the indicators in the motor operation
        gStepper_Obj.RUN_STEPPER = false;
        gStepper_Obj.MANUAL_ADVANCE = false;
        gStepper_Obj.RECIPROCATION = false;
        gStepper_Obj.MOTOR_CMD_ACTIVE1 = false;
        gStepper_Obj.MOTOR_CMD_ACTIVE2 = false;
        gStepper_Obj.MOTOR_CMD_ACTIVE3 = false;
        // stop the timer also
        TBCTL &=  ~MC_3;                    // Stop timer by setting mc_0
        TBCCR0 = 0x0000;                    // once stopped reset the timer value
        // clear temp_reciprocation and MoveSteps so next movement is correct
        temp_reciprocation = false;         // reset reciprocation
        MoveSteps = false;
        gStepper_Obj.AccelerateState = IDLE; // set AccelerateState to idle when
                    // sleeping or disabled
    }
    if (gStepper_Obj.IFS != IFS_previous)
    {
        DAC12_0DAT = 1241*gStepper_Obj.IFS;
        IFS_previous = gStepper_Obj.IFS;
        _nop();
    }
}

//
// routine to determine if any changes are in motor operation are required
// and pass the information to the motor state machine
//
void UpdateSpeed()
{
    StartingSpeed = (unsigned long) gStepper_Obj.STARTING_SPEED << use_one_edge;       // Configure the starting speed
    StoppingSpeed = (unsigned long) gStepper_Obj.STOPPING_SPEED << use_one_edge;       // Configure the stopping speed
    StepsToMove   = (unsigned long) gStepper_Obj.NUM_STEPS << use_one_edge;            // adjust for both edges
    StepsToStop   = (unsigned long) gStepper_Obj.STEPS_2_STOP << use_one_edge;         // adjust for both edges
    DesiredTargetSpeed = (unsigned long) gStepper_Obj.TARGET_SPEED << use_one_edge;    // Configure the target speed
    AccelRate     = (unsigned long) gStepper_Obj.ACCEL_RATE << use_one_edge;           // Configure the acceleration rate here

    DetermineMotorMovement();                               // Once type of motor movement is determined
    MotorStateMachine();                                    // Command the motor to move
}

//
//  Routine to determine desired motor operation
//  direction, acceleration, deceleration, steps to move
//  reciprocation
//
void  DetermineMotorMovement()
{
    // calculate steps to target and steps to stop prior to starting motor
    if ((gStepper_Obj.RUN_STEPPER == true || gStepper_Obj.MANUAL_ADVANCE == true || gStepper_Obj.RECIPROCATION == true) && \
         gStepper_Obj.MOTOR_CMD_ACTIVE2 == false && gStepper_Obj.MOTOR_CMD_ACTIVE1 == false && gStepper_Obj.MOTOR_CMD_ACTIVE3 == false )
    {
        // update steps to speed and steps to stop calculations
        // only calculate when the motor is idle and commanded to move
        time = (float)(DesiredTargetSpeed - StartingSpeed)/AccelRate;
        time_squared = time * time;
        // calculate steps to target
        Steps2Target = ((AccelRate >> 1) * (time_squared)) + (StartingSpeed * time);
        // calculate Steps2Stop if:
        //  StepsToStop in GUI is >1 and
        //  StoppingSpeed is not equal Starting Speed
        //
        if ((StepsToStop == 2 && use_one_edge == 1) || StepsToStop == 1)
        {
            if (StartingSpeed != StoppingSpeed)
            {   // recalculate time if necessary
                time = (float)(DesiredTargetSpeed - StoppingSpeed)/AccelRate;
                time_squared = time * time;
            }
            // calculate steps required to stop
                Steps2Stop = ((AccelRate >> 1) * (time_squared)) + (StartingSpeed * time);
        }
        else    // defined number of steps to stop
        {
            Steps2Stop = StepsToStop;
        }
        // calculate stepping rate update (integer value divide by 32)
        if (AccelRate > 31 || AccelRate == 0)                   // if >=32 or 0
        {
            SteppingRateUpdate = AccelRate >> 5;                // divide by 32 - right shift 5
        }
        else
        {
            SteppingRateUpdate = 1;                             // increment by 1 to get correct value
        }
        if ((StartingSpeed >= DesiredTargetSpeed) || (StoppingSpeed >= DesiredTargetSpeed))
            // Special case -- override prior settings
            // allow motor to attempt to run only at starting speed
        {
            SteppingRateUpdate = 0;
            Steps2Target = 0;
            Steps2Stop = 0;
        }
        // transition from ACCEL to DECEL when count equals (StepsToMove - Steps2Stop) <=0
        if ((Steps2Stop > (StepsToMove >>1 )) && (StartingSpeed == StoppingSpeed))
            Steps2Decel = StepsToMove >>1;                      // decel at midpoint
        else
            Steps2Decel = StepsToMove - Steps2Stop;
    }
    // start running or stopping the motor if commanded and not manually advancing
    if (gStepper_Obj.RUN_STEPPER == true && gStepper_Obj.MOTOR_CMD_ACTIVE1 == false && gStepper_Obj.MOTOR_CMD_ACTIVE2 == false && gStepper_Obj.MOTOR_CMD_ACTIVE3 == false)
    {
        // START STEPPER
        gStepper_Obj.MOTOR_CMD_ACTIVE1 = true;                 // Indicate that the motor is running

        LastCommandWasRunSteps = true;              // flag to determine if the timer requires clearing

        DesiredStepperSpeed = DesiredTargetSpeed;   // Configure the final stepping rate in PPS

        SteppingRate = StartingSpeed;               // This is the starting speed from the GUI
        // adjusted by the edge indicator
        SteppingRateTMR = SMCLK_FREQ/StartingSpeed; // Convert here to the timer value -- xxTMR means the
        // value to be loaded into the timer
        gStepper_Obj.AccelerateState = ACCEL;                    // signal to start state machine
        // Configure TimerA
        TACCTL1 &= 0xFF1F;                          // Clear OUTMODx bits; 3 MSB on lower byte; Timer configured to output mode
        TACCTL1 |= OUTMOD_4 + CCIE;                 // Configure the timer as TOGGLE. TBCCR0 does the reset
        TACCR0 = TIME_32ms;                         // set timer to 32ms
        TACCR1 = TIME_32ms >> 1;
        // configure TimerB
        TBCCTL2 &= 0xFF1F;                          // Clear OUTMODx bits; 3 MSB on lower byte; Timer configured to output mode
        TBCCTL2 |= OUTMOD_4 + CCIE;
        TBCCR0 = SteppingRateTMR;                   // Start the timer here with initial value
        TBCCR2 = SteppingRateTMR >> 1;              // set TBCCR1 to the midpoint of the TBCCR0 for toggle
        // start timers
        TACTL |= MC_1;                              // Start timerA by selecting up mode
        TBCTL |= MC_1;                              // start timerB

    }
    if ((gStepper_Obj.RUN_STEPPER == false && gStepper_Obj.MOTOR_CMD_ACTIVE1 == true))
        // motor is running and stop is issued
    {
        // STOP STEPPER
        gStepper_Obj.MOTOR_CMD_ACTIVE1 = false;                // must set low when stopping
        gStepper_Obj.AccelerateState = DECEL;
        DesiredStepperSpeed = StoppingSpeed;        //Speed to stop at
    }
    // MOVE_STEPS
    // Manually advance steps if commanded
    // only allow starting if not automatically running
    if (gStepper_Obj.MANUAL_ADVANCE == true && gStepper_Obj.MOTOR_CMD_ACTIVE1 == false && gStepper_Obj.MOTOR_CMD_ACTIVE2 == false && gStepper_Obj.MOTOR_CMD_ACTIVE3 == false)
    {
        gStepper_Obj.MOTOR_CMD_ACTIVE2 = true;
        if (LastCommandWasRunSteps == true || use_one_edge == true)         // 15APR -- added control to avoid resetting
        {
            // reset timer only when starting or when using one edge
            TBCTL |= TBCLR;                         // reset timerA
            TBCTL = CNTL_0 | TBSSEL_2 |ID_0 | MC_0;
            TBCCTL2 &= 0xFF1F;                      // Clear OUTMODx bits; 3 MSB on lower byte; Timer configured to output mode
            TBCCTL2 |= OUTMOD_4 + CCIE;             // Configure the timer as SET_RESET. TBCCR0 does the reset
        }
        LastCommandWasRunSteps = false;             // reset flag to indicate move steps was last commanded
        DesiredStepperSpeed = DesiredTargetSpeed;   // Configure the Frequency Rate

        SteppingRate = StartingSpeed;
        SteppingRateTMR = SMCLK_FREQ/StartingSpeed; // convert to timer value
        gStepper_Obj.AccelerateState = ACCEL;                    // signal to start state machine
        tmpStepsToMove = 0;                         // clear counter
        MoveSteps = true;                           // set flag to indicate manual advance

        // Configure TimerA
       TACCTL1 &= 0xFF1F;                          // Clear OUTMODx bits; 3 MSB on lower byte; Timer configured to output mode
       TACCTL1 |= OUTMOD_4 + CCIE;                 // Configure the timer as TOGGLE. TBCCR0 does the reset
       TACCR0 = TIME_32ms;                         // set timer to 32ms
       TACCR1 = TIME_32ms >> 1;
       // configure TimerB
       TBCCR0 = SteppingRateTMR;                   // Start the timer here with initial value
       TBCCR2 = SteppingRateTMR >> 1;              // set TBCCR1 to the midpoint of the TBCCR0 for toggle

        // start timers
        TACTL |= MC_1;                              // Start timer by selecting up mode
        TBCTL |= MC_1;

    }

    if(gStepper_Obj.RECIPROCATION == true && gStepper_Obj.MANUAL_ADVANCE == true && temp_reciprocation == false)
    {
        last_reciprocation_state = 1;
        temp_reciprocation = true;
        gStepper_Obj.MOTOR_CMD_ACTIVE3 = true;
        if (LastCommandWasRunSteps == true || use_one_edge == true)         // 15APR -- added control to avoid resetting
        {
            // reset timer only when starting or when using one edge
            TBCTL |= TBCLR;                             // reset timerA
            TBCTL = CNTL_0 | TBSSEL_2 |ID_0 | MC_0;
            TBCCTL2 &= 0xFF1F;                          // Clear OUTMODx bits; 3 MSB on lower byte; Timer configured to output mode
            TBCCTL2 |= OUTMOD_4 + CCIE;                 // Configure the timer as SET_RESET. TBCCR0 does the reset
        }
            LastCommandWasRunSteps = false;             // reset flag to indicate move steps was last commanded
            DesiredStepperSpeed = DesiredTargetSpeed;   // Configure the Frequency Rate

            SteppingRate = StartingSpeed;
            SteppingRateTMR = SMCLK_FREQ/StartingSpeed; // convert to timer value
            __delay_cycles(10000000);
            gStepper_Obj.AccelerateState = ACCEL;       // signal to start state machine
            tmpStepsToMove = 0;                         // clear counter
            MoveSteps = true;                           // set flag to indicate manual advance
            // Configure TimerA
           TACCTL1 &= 0xFF1F;                          // Clear OUTMODx bits; 3 MSB on lower byte; Timer configured to output mode
           TACCTL1 |= OUTMOD_4 + CCIE;                 // Configure the timer as TOGGLE. TBCCR0 does the reset
           TACCR0 = TIME_32ms;                         // set timer to 32ms
           TACCR1 = TIME_32ms >> 1;
           // configure TimerB
           TBCCR0 = SteppingRateTMR;                   // Start the timer here with initial value
           TBCCR2 = SteppingRateTMR >> 1;              // set TBCCR1 to the midpoint of the TBCCR0 for toggle
           // start timers
            TACTL |= MC_1;                              // Start timer by selecting up mode
            TBCTL |= MC_1;

    }
    // Special case of updating speed during running
    // if the Desired Target Speed has been updated, then the stepper speed should be increased or decreased
    //
    if ((gStepper_Obj.RUN_STEPPER == true) && (DesiredStepperSpeed != DesiredTargetSpeed))
    {
        if (DesiredTargetSpeed > DesiredStepperSpeed)
        {
            gStepper_Obj.AccelerateState = ACCEL;

        }
        else
            gStepper_Obj.AccelerateState = DECEL2NEWTARGET;
        // reset DesiredStepperSpeed
        DesiredStepperSpeed = DesiredTargetSpeed;
    }
}

// Stepper State machine
// This is used to move the motor
void MotorStateMachine()
{
    // update the SteppingRateTMR if required
    // reduce the number of calculations
    if (UpdateSteppingRateTMR == true)
    {
        SteppingRateTMR = SMCLK_FREQ/SteppingRate;  // calculate new value
        UpdateSteppingRateTMR = false;              // clear flag
    }
    switch (gStepper_Obj.AccelerateState)
    {
    case (IDLE):
            if(gStepper_Obj.RECIPROCATION == false && gDRV843x_Obj.nSleep == 1 && gDRV843x_Obj.gEnable == 1)
            {
                gStepper_Obj.MOTOR_CMD_ACTIVE1 = false;
                gStepper_Obj.MOTOR_CMD_ACTIVE2 = false;
                gStepper_Obj.MOTOR_CMD_ACTIVE3 = false;
                gStepper_Obj.MANUAL_ADVANCE = false;           // clear the flags once stops
                MoveSteps = false;                  // clear the MoveSteps flag here
                // reset the timers if the last command was run motor
                if (LastCommandWasRunSteps == true)
                {
                    TACTL &= ~MC_3;                     // Stop timerA
                    TBCTL &= ~MC_3;                     // Stop timerB
                }
                if((gStepper_Obj.RECIPROCATION == true && gStepper_Obj.MANUAL_ADVANCE == true))
                {
                    __delay_cycles(10000000);   // pause here if reciprocating
                }
            }
    break;
    case (ACCEL):
        {
        // do nothing in ACCEL mode, wait for external input to change state
        }
    break;
    case (DECEL):
        {
        // do nothing in DECEL mode, wait for external input to change state
        }
    break;
    case (RUN):
        {
        // do nothing in RUN mode, wait for external input to change state
        }
    break;
    case (DECEL2NEWTARGET):
        {
        // do nothing in DECEL mode, wait for external input to change state
        }
    break;
    }
}

