/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *
 *  stepper.h - Declaration file for functions and global variables related to stepper control
 *  DRV843x_EVM_FIRMWARE
 *  10/31/2020
 *
 ******************************************************************************/
#ifndef STEPPER_H_
 #define STEPPER_H_
/******************************************************************************
 * HEADER FILES
 *****************************************************************************/
#include <dataTypeDefinition.h>
#include <drv84xx/include/drv84xx.h>
#include <msp430.h>
#include <stepper.h>
#include "msp430f2617.h"
/******************************************************************************
 * Defined Quantities
 *****************************************************************************/
#define LED_TIMEOUT 200                     // Count to toggle LED
#define SMCLK_FREQ  8000000                 // SMCLK frequency in Hz
#define TIME_32ms   31250                   // TimerA interrupt value, (31.25ms or 32 updates/s)

#define DEFAULT_START_SPEED     2000        // initial speed in pulses per second when starting motor
#define DEFAULT_TARGET_SPEED    10000       // speed of motor once running
#define DEFAULT_ACCEL_RATE      1000        // acceleration rate from starting speed to target speed
#define DEFAULT_STOPPING_SPEED  2000        // speed to stop motor once deceleration has reached this point
#define DEFAULT_NUM_STEPS       3200        // number of steps to move when manually stepping
#define DEFAULT_STEPS_2_STOP    0           // number of steps used to stop the motor when manually stepping
#define DEFAULT_IFS             1.1

#define use_one_edge            1           // multiplies all step calculations by 2
                                            // this is legacy from the DRV8846
// defines for motor state machine
#define     IDLE            1
#define     ACCEL           2
#define     DECEL           4
#define     RUN             8
#define     DECEL2NEWTARGET 16

/*****************************************************************************/
// Declare Global Variables

//***********************************************************************************
// Global Variables for GUI Composer                                                         *
//***********************************************************************************

typedef struct Stepper_Driving_Obj
{
   bool MOTOR_CMD_ACTIVE1;     // Motor in Motion (Auto)   0 - No Motion   1 - In Motion
   bool MOTOR_CMD_ACTIVE2;     // Motor in Motion (Man)    0 - No Motion   1 - In Motion
   bool MOTOR_CMD_ACTIVE3;
   bool RUN_STEPPER;           // Auto run enabled
   bool MANUAL_ADVANCE;        // Manually advance motor
   bool RECIPROCATION;         // Reciprocation in Step mode

   volatile uint8_t gDIR;               // Direction                0 - Forward     1 - Reverse

   uint16_t  STARTING_SPEED;    // initial speed in pulses per second when starting motor
   uint16_t  TARGET_SPEED;      // speed of motor once running
   uint16_t  ACCEL_RATE;        // acceleration rate from starting speed to target speed
   uint16_t  STOPPING_SPEED;    // speed to stop motor once deceleration has reached this point
   uint16_t  STEPS_2_STOP;      // number of steps used to stop the motor when manually stepping
   uint16_t  NUM_STEPS;         // number of steps to move when manually stepping
   uint16_t  AccelerateState;   // Accelerate state for state machine

   float  IFS;                  // Full scale current

} Stepper_Driving_Obj_t;

extern volatile Stepper_Driving_Obj_t gStepper_Obj;

/////////////////////////
// variables for utility
/////////////////////////

// variable for the motor state machine
unsigned long int   SteppingRateTMR;
unsigned long int   SteppingRate;
unsigned long int   SteppingRateUpdate;
unsigned long int   DesiredStepperSpeed;
unsigned long int   DesiredTargetSpeed;
unsigned long int   StartingSpeed;
unsigned long int   StoppingSpeed;
unsigned long int   AccelRate;
unsigned long int   StepsToStop;
unsigned long int   StepsToMove;
unsigned long int   tmpStepsToMove;
unsigned long int   PriorSteppingRate;

bool             temp_reciprocation;
bool             LastCommandWasRunSteps;
bool             MoveSteps;
bool             UpdateSteppingRateTMR;

float               time;
float               time_squared;
float               IFS_previous;        // Previous full-scale current setting

unsigned int        last_reciprocation_state;       // reciprocation flag, =1 last action is reciprocation
unsigned int        LED_TOGGLE_CNT;                 // count to determine if LED should be toggled
unsigned int        MICROSTEP_MODE_BITS;            // Microstep mode bits to store microstepping setting

unsigned long int   Steps2Target;       // number of steps to reach target speed
                                        // from starting speed using acceleration rate
unsigned long int   Steps2Stop;         // number of steps to stop
                                        // from target speed using acceleration rate
unsigned long int   Steps2Decel;        // step number to change from accel to decel
                                        // in manual mode

unsigned int        clear_fault_loop;

/*****************************************************************************/

// Function Declarations

// Main Functions
void UpdateDeviceControls();
void UpdateGUIControls();
void UpdateSpeed();

// GPIO/GUI Functions
void stepper_Init();
void SetDefaultGUIVals();

// Motor Functions
void DetermineMotorMovement();
void MotorStateMachine();
void DetermineMicrostepping(char CTRL3Data);

#endif /*STEPPER_H_*/
